library ieee;
use ieee.std_logic_1164.all;


entity edge_detect is
  port(
    clk, reset: in std_logic;
    level: in std_logic;
    tick: out std_logic
  );
end edge_detect;


-- Listing 5.4 from Textbook
architecture MEALY_ARCHITECTURE of edge_detect is
  type state_type is (S0, S1);
  signal state_reg, state_next : state_type;
  begin
  
  -- state register; process #1
  process (clk , reset)
  begin
    if (reset = '1') then 
      state_reg <= S0;
    elsif (clk' event and clk = '1') then 
      state_reg <= state_next;
    end if;
  end process;
  
  -- next state and output logic; process #2
  process (state_reg, level)
  begin
    state_next <= state_reg;
    tick <= '0';
    case state_reg is 
      when S0 =>
        if level = '1' then
          state_next <= S1;
          tick <= '1';
        end if;
      when S1 =>
        if level = '0' then
          state_next <= S0;
        end if;
    end case;
  end process;
end MEALY_ARCHITECTURE;  


-- Listing 5.3 from Textbook
architecture MOORE_ARCHITECTURE of edge_detect is
  type state_type is (zero, edge, one);
  signal state_reg, state_next: state_type;
begin

  -- state register
  process(clk,reset)
  begin
    if (reset='1') then
      state_reg <= zero;
    elsif (clk'event and clk='1') then
      state_reg <= state_next;
    end if;
  end process;
   
  -- next-state/output logic
  process(state_reg,level)
  begin
    state_next <= state_reg;
    tick <= '0';
    case state_reg is
      when zero=>
        if level= '1' then
          state_next <= edge;
        end if;
      when edge =>
        tick <= '1';
        if level= '1' then
          state_next <= one;
        else
          state_next <= zero;
        end if;
      when one =>
        if level= '0' then
          state_next <= zero;
        end if;
    end case;
  end process;
end MOORE_ARCHITECTURE;
